from  pylablib.devices import Thorlabs
from time import sleep
import configparser

class Stage():
    config_file = "src/configs/stage_config.ini"
    open = False

    def __init__(self):
        try:
            self.motor_x = Thorlabs.KinesisMotor("80855672")
            self.motor_y = Thorlabs.KinesisMotor("80855670")
            self.motor_z = Thorlabs.KinesisMotor("80855667")
            self.motor_x.close()
            self.motor_y.close()
            self.motor_z.close()
            self.open = True
        except:
            self.open = False


        self.min_step = 46694 / 10
        self.open = False
        self.offset_x = 0
        self.offset_y = 0
        self.offset_z = 0

        self.load_config()

    def load_config(self):
        config = configparser.ConfigParser()
        config.read(self.config_file)
        if "Stage" in config:
            section = config["Stage"]
            self.offset_x = float(section.get("offset_x", self.offset_x))
            self.offset_y = float(section.get("offset_y", self.offset_y))
            self.offset_z = float(section.get("offset_z", self.offset_z))

    def save_config(self):
        config = configparser.ConfigParser()
        config["Stage"] = {
            "offset_x": str(self.offset_x),
            "offset_y": str(self.offset_y),
            "offset_z": str(self.offset_z)
        }
        with open(self.config_file, "w") as config_file:
            config.write(config_file)

    def __del__(self):
        self.close_connections()

    def stop_all(self):
        self.motor_x.stop()
        self.motor_y.stop()
        self.motor_z.stop()

    def enable_connections(self):
        if not self.open:
            self.motor_x = Thorlabs.KinesisMotor("80855672")
            self.motor_y = Thorlabs.KinesisMotor("80855670")
            self.motor_z = Thorlabs.KinesisMotor("80855667")
            self.open = True

    def close_connections(self):
        if self.open:
            self.motor_x.close()
            self.motor_y.close()
            self.motor_z.close()
            self.open = False

    def home_axis(self):
        if self.open:
            self.offset_x = self.motor_x.get_position(scale=False)
            self.offset_y = self.motor_y.get_position(scale=False)
            self.offset_z = self.motor_z.get_position(scale=False)
            self.save_config()


    def wait_stable(self):
        if self.open:
            self.motor_x.wait_move
            self.motor_y.wait_move
            self.motor_z.wait_move
            sleep(0.5)

    #################################################################
    # MOVE MOTORS BY A NUMBER OF STEPS
    #################################################################

    def move_x(self, nb_steps):
        if not self.motor_x.is_moving() and self.open:
            self.motor_x.move_by(nb_steps*self.min_step, scale=False) 
            self.motor_x.wait_move()

    def move_y(self, nb_steps):
        if not self.motor_y.is_moving() and self.open:
            self.motor_y.move_by(-nb_steps*self.min_step, scale=False) 
            self.motor_y.wait_move()

    def move_z(self, nb_steps):
        if not self.motor_z.is_moving() and self.open:
            self.motor_z.move_by(nb_steps*self.min_step, scale=False) 
            self.motor_z.wait_move()

    #################################################################
    # MOVE MOTORS TO A GIVEN POSITION
    #################################################################

    def move_x_to(self, position):
        if self.open:
            self.motor_x.move_to(position*46694 + self.offset_x, scale=False)
            self.motor_x.wait_move()

    def move_y_to(self, position):
        if self.open:
            self.motor_y.move_to(position*46694 + self.offset_y, scale=False)
            self.motor_y.wait_move()

    def move_z_to(self, position):
        if self.open:
            self.motor_z.move_to(position*46694 + self.offset_z, scale=False)
            self.motor_z.wait_move()

    #################################################################
    # JOG
    #################################################################

    def jog_x(self, dir):
        if not self.motor_x.is_moving() and self.open:
            self.motor_x.jog(dir)

    def jog_y(self, dir):
        if not self.motor_y.is_moving() and self.open:
            self.motor_y.jog(dir)

    def jog_z(self, dir):
        if not self.motor_z.is_moving() and self.open:
            self.motor_z.jog(dir)

    #################################################################
    # GET THE POSITION
    #################################################################

    def get_x_pos_str(self) -> str:
        if self.open:
            return f"{(self.motor_x.get_position(scale=False) - self.offset_x) / 46694.:.{3}f}mm"
        else:
            return "?"
    
    def get_y_pos_str(self) -> str:
        if self.open:
            return f"{(self.motor_y.get_position(scale=False) - self.offset_y) / 46694.:.{3}f}mm"
        else:
            return "?"
    
    def get_z_pos_str(self) -> str:
        if self.open:
            return f"{(self.motor_z.get_position(scale=False) - self.offset_z) / 46694.:.{3}f}mm"
        else:
            return "?"
        
    def get_x_pos(self) -> float:
        if self.open:
            return (self.motor_x.get_position(scale=False) - self.offset_x) / 46694.
        else:
            return float('nan')
    
    def get_y_pos(self) -> float:
        if self.open:
            return (self.motor_y.get_position(scale=False) - self.offset_y) / 46694.
        else:
            return float('nan')
    
    def get_z_pos(self) -> float:
        if self.open:
            return (self.motor_z.get_position(scale=False) - self.offset_z) / 46694.
        else:
            return float('nan')